/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.auth;

import java.util.Set;

import org.jvnet.libpam.PAM;
import org.jvnet.libpam.PAMException;
import org.jvnet.libpam.UnixUser;

/**
 * PAMAuthentication will authenticate user to local unix machine using PAM.
 * This class is a wrapper of libpam4j, a Java/PAM bindings using JNA, to use internally in 4ECaps.
 */
public class PAMAuthentication {

	/**
	 * Check whether user is exist
	 * @param username Username
	 * @return true if user is exist
	 */	
	public static boolean isExist(String username) {
		return UnixUser.exists(username);
	}	
	
	/**
	 * Get current user logged in Unix system
	 * @return username of user logged in
	 */	
	public static String getCurrentUserLoggedIn() {
		return System.getProperty("user.name");
	}
	
	/**
	 * Get username of current user logged in
	 * @return username of user logged in
	 * @throws PAMException
	 */	
	public static String getUsername() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getUserName();		
	}
	
	/**
	 * Get UID of current user logged in
	 * @return UID of user logged in
	 * @throws PAMException
	 */		
	public static int getUID() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getUID();		
	}
	
	/**
	 * Get GID of current user logged in
	 * @return GID of user logged in
	 * @throws PAMException
	 */		
	public static int getGID() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getGID();		
	}	
	
	/**
	 * Get gecos (the real name) of current user logged in
	 * @return gecos of user logged in
	 * @throws PAMException
	 */		
	public static String getGecos() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getGecos();		
	}		
	
	/**
	 * Get home directory of current user logged in
	 * @return home directory of user logged in
	 * @throws PAMException
	 */			
	public static String getDir() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getDir();		
	}	
	
	/**
	 * Get the shell of current user logged in
	 * @return the shell of user logged in
	 * @throws PAMException
	 */			
	public static String getShell() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getShell();		
	}		
	
	/**
	 * Get the groups that user logged in belongs to
	 * @return the groups of user logged in
	 * @throws PAMException
	 */		
	public static Set<String> getGroups() throws PAMException {
    	UnixUser ux = new UnixUser(getCurrentUserLoggedIn());
    	return ux.getGroups();		
	}		
	
	/**
	 * Authenticate user
	 * @param service Service
	 * @param username Username
	 * @param password Password 
	 * @return authenticated user in UnixUser object
	 * @throws PAMException
	 */		
	public static UnixUser authenticate(String service, String username, String password) throws PAMException {
		PAM p = new PAM(service);
		return p.authenticate(username, password);		
	}
	
	
}


 
