/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.auth;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.security.MessageDigest;

import org.apache.commons.codec.binary.Hex;

/**
 * SuperUserAuthentication provides mechanism to store password into a file.
 * This mechanism is implemented in superuser authentication of an application.
 * The config file containing the path of saved password located in the resources folder.
 */
public class SuperUserAuthentication {
	
	/**
	 * Private constructor to prevent instantiation.
	 */		
	private SuperUserAuthentication()
	{
	}	

	/**
	 * Password filename that will be created in the folder path.
	 */		
	private static String passwordFilename = "superuser.pwd";
	
	
	/**
	 * Validate password whether it equals with the encrypted file, if file is not exist then set the password by creating it.
	 * @param pwd Password
	 * @return true if process succeed
	 * @throws Exception
	 */
	public static boolean validateOrSet(String pwd) throws Exception {
		File file=getPwdFile();
		String encpwd=null;
		boolean rtn=false;
		if(file.exists()){
			encpwd=readFile(file);
		}
		if(encpwd!=null && !encpwd.equals("")){
			rtn=isValid(pwd,encpwd);
		}else{
			setPassword(pwd);
			rtn=true;
		}
		return rtn;
	}
	
	
	/**
	 * Change password to a new one if password is valid 
	 * @param pwd Password
	 * @param newpassword New password
	 * @return true if process succeed 
	 * @throws Exception
	 */
	public static boolean changePwd(String pwd,String newpassword) throws Exception {
		 boolean rtn=validateOrSet(pwd);
		 
		 if(rtn){
			 //Thread.sleep(1000); //wait till it closes the stream ....
			 setPassword(newpassword);			 
		 }
		return rtn;
	}
	
	
	/**
	 * Check whether password is valid, compared to encrypted password
	 * @param password Password
	 * @param encpass Encrypted password
	 * @return true if encrypted password equals salt password 
	 * @throws Exception
	 */	
	private static boolean isValid(String password, String encpass) throws Exception {
		String salt=encpass.split(":")[2];
		String saltpass=":B:"+salt+":"+getMD5(salt+"-"+getMD5(password));
		if(encpass.equals(saltpass)){
			return true;
		}else{
			return false;
		}
	}
	
	
	/**
	 * Get MD5 of source string
	 * @param source Source string
	 * @return MD5 string 
	 * @throws Exception
	 */		
	private static String getMD5(String source) throws Exception {
		
		byte[] bytesOfMessage = source.getBytes();

		java.security.MessageDigest md = MessageDigest.getInstance("MD5");
		byte[] thedigest = md.digest(bytesOfMessage);
		 
		final String result = new String(Hex.encodeHex(thedigest));
		return result;
	}
	
	
	/**
	 * Set password to encrypted file
	 * @param plain_pwd Plain password
	 * @throws Exception
	 */			
	private static void setPassword(String plain_pwd) throws Exception {
		BufferedWriter writer = null;
		try {
			String salt="d1e93dec";
			String saltpass=":B:"+salt+":"+getMD5(salt+"-"+getMD5(plain_pwd));		
			File file=getPwdFile();
			writer = new BufferedWriter(new FileWriter(file));
			writer.write(saltpass);
		}
		finally {
			if (writer != null) writer.close();		
		}
		
	}
	
	
	/**
	 * Get password file
	 * @return file
	 * @throws Exception
	 */			
	private static File getPwdFile() throws Exception {
		String root = Utils.GetConfigValue("superuser_password_folder");
		File file =new File(root + passwordFilename);
		if ( !file.exists() ) {
			file.createNewFile();
		}
		return file;
	}
	
	
	/**
	 * Read file
	 * @param path Path file
	 * @throws IOException
	 */		
	private static String readFile(File path) throws IOException {
		StringBuilder contents = new StringBuilder();
		BufferedReader input = null;
	    try {
	    	input = new BufferedReader(new FileReader(path));
    		String line = input.readLine(); 
    		if (line != null) {
    			contents.append(line);
    		}
	    }
    	finally {
    		if (input != null) input.close();
    	}	    
	    return contents.toString();
	}	
	
}


 