/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.auth;

import java.security.MessageDigest;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.codec.binary.Hex;

/**
 * WikiAuthentication will check / validate user authentication to Wiki database.
 * It also provide functions to get encrypted password of specific user & get list of Wiki users.
 */
public class WikiAuthentication {
	
	public static final String SESSION_LOGGED_USER = "session^logged^user";
	
	/**
	 * Private constructor to prevent instantiation.
	 */		
	private WikiAuthentication()
	{
	}		

	/**
	 * Validate Wiki User. If user_name is valid & password being encrypted equals value in database then it will return true. 
	 * @param user_name
	 * @param password
	 * @return is user valid
	 * @throws Exception
	 */		
	public static boolean validateUser(String user_name, String password) throws Exception {
		String encpass = getEncryptedPwd(user_name);
		return isValid(password,encpass);
	}
	
	/**
	 * Validate Wiki User With Encrypted Password. If user_name is valid & encrypted password equals value in database then it will return true. 
	 * @param user_name
	 * @param encryptedPassword
	 * @return is user valid
	 * @throws Exception
	 */		
	public static boolean validateUserEncryptedPassword(String user_name, String encryptedPassword) throws Exception {
		String encpass = getEncryptedPwd(user_name);
		if (encryptedPassword == null) {
			return false;
		}
		return (encryptedPassword.equals(encpass));
	}	
	
	
	/**
	 * Check whether password is valid, compare to encrypted password
	 * @param password Password
	 * @param encpass Encrypted password
	 * @return true if encrypted password equals salt password
	 * @throws Exception
	 */	
	private static boolean isValid(String password, String encpass) throws Exception {
		if (encpass == null || password == null) return false;
		String salt=encpass.split(":")[2];
		String saltpass=":B:"+salt+":"+getMD5(salt+"-"+getMD5(password));
		if(encpass.equals(saltpass)){
			return true;
		}else{
			return false;
		}
	}
	
	
	/**
	 * Get MD5 of source string
	 * @param source Source string
	 * @return MD5 string 
	 * @throws Exception
	 */		
	private static String getMD5(String source) throws Exception {
		
		byte[] bytesOfMessage = source.getBytes();

		java.security.MessageDigest md = MessageDigest.getInstance("MD5");
		byte[] thedigest = md.digest(bytesOfMessage);
		 
		final String result = new String(Hex.encodeHex(thedigest));
		return result;
	}
	
	
	/**
	 * List / get all Wiki users. The fields returned are : user_name, user_real_name, user_password, user_email.
	 * @return users
	 * @throws Exception
	 */	
	public static Vector<HashMap<String, String>> listUsers() throws Exception {
		
		Vector<HashMap<String, String>> rtn = null;		
		
		DBManager dbm = new DBManager("wiki");
		dbm.connect();

		//sql = "select user_name,user_real_name,user_password,user_email from user";
		
		ArrayList<String> selectedFields = new ArrayList<String>();
		selectedFields.add("user_name");
		selectedFields.add("user_real_name");
		selectedFields.add("user_password");
		selectedFields.add("user_email");
		
		ResultSet rs = dbm.GetDatabase("user", selectedFields, null, null);
		
		rtn = new Vector<HashMap<String, String>>();
		
		while(rs.next()){			
			String user_name=rs.getString("user_name");
			String user_real_name=rs.getString("user_real_name");
			String user_password=rs.getString("user_password");
			String user_email=rs.getString("user_email");
			HashMap<String, String> h=new HashMap<String, String>();
			h.put("user_name",user_name );
			h.put("user_real_name",user_real_name );
			h.put("user_password",user_password );
			h.put("user_email",user_email );
			rtn.add(h);
		}
		dbm.closeConnection();
		
		return rtn;	
	}		
	
	
	/**
	 * Get encrypted password of specific user
	 * @param user
	 * @return encrypted password
	 * @throws Exception
	 */		
	public static String getEncryptedPwd(String user) throws Exception {
		
		if ((user == null) || user.equals("")) return null;
		
		String rtn = null;		
		
		DBManager dbm = new DBManager("wiki");
		dbm.connect();
		
		//sql = "select CAST(user_password AS CHAR(10000) CHARACTER SET utf8) as pass from user where user_name=?"

		ArrayList<String> selectedFields = new ArrayList<String>();
		selectedFields.add("CAST(user_password AS CHAR(10000) CHARACTER SET utf8) as pass");
		
		Map<String,Object> queryParams = new HashMap<String,Object>();
		queryParams.put("user_name", Utils.capitalizeString(user));
		
		ResultSet rs = dbm.GetDatabase("user", selectedFields, queryParams, null);
		
		while(rs.next()){			
			rtn=rs.getString("pass");
		}
		dbm.closeConnection();
	
		return rtn;	
	}	
	
}

 
