/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * The DBManager class provides an Access Interface between the Trading Database and Java.
 * It can be used to get data from the database, insert new data and update existing data.
 * The config file containing the details for the database hostname and credentials is located in the resources folder.
 * @author Manas Maral Misra, Antonius Ari Wicaksono
 *
 */
public class DBManager {
	/**
	 * The database name for the current instance of DBManager
	 */
	private String dbName;
	/**
	 * The connection string for the database connection. Stored in config.lib-dal
	 */
	private String connectionString;
	/**
	 * The resultSet which contains the returned results from the SQL Query of GetDatabase
	 */
	ResultSet resultSet = null;
	/**
	 * The connection object of the current connection.
	 */
	private Connection conn = null;

	/**
	 * Initialize the DBManager with the database name. 
	 * @param dbName The name of the database e.x. trading, tradingRef, fundamentals
	 * @throws IOException
	 */
	public DBManager(String dbName) throws IOException
	{
		this.dbName = dbName;
	}

	/**
	 * This function connects to the dbName database;
	 * @throws SQLException This is thrown incase we are not able to connect to the database;
	 * @throws ClassNotFoundException This is thrown incase the driver is not found on this machine
	 * @throws IOException This is thrown incase the config.lib-dal if not found on this machine under resources folder
	 */
	public void connect() throws SQLException, ClassNotFoundException, IOException
	{
		String connString = Utils.getConfig4E(".CONFIG4E_JDBC_CONNECTIONSTRING_" + dbName.toUpperCase());
		
		String driver = "com.mysql.jdbc.Driver";
		
		// if connString contains 'sqlserver'. i.e. 'jdbc:sqlserver://10.153.64.3:1433;databaseName=infrastructure;integratedSecurity=false;user=dbuser;password=dbuser'
		if (connString.contains("sqlserver")) {
			driver="com.microsoft.sqlserver.jdbc.SQLServerDriver"; 
		}

		Class.forName(driver);
		conn = DriverManager.getConnection(connString);
		connectionString = conn.toString();
	}
	
	/**
	 * This function returns the result set of the SQL Query used by GetDatabase;
	 * @param query The SQL Query passed to it.
	 * @return
	 * @throws SQLException
	 */
	protected ResultSet executeQuery(String query) throws SQLException
	{
		Date start = new Date();
		Statement statement = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
		resultSet = statement.executeQuery(query);
		Date end = new Date();
		long duration = (end.getTime() - start.getTime());
		//Utils.Log("DEBUG: Query ("+query+") took "+duration+ " miliseconds");
		return resultSet;
	}
	/**
	 * This function returns the result set of the SQL Query used by GetDatabase;
	 * @param query The SQL Query passed to it.
	 * @return
	 * @throws SQLException
	 */
	protected void executeNonQuery(String query) throws SQLException
	{
		Date start = new Date();
		Statement statement = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
		statement.execute(query);
		Date end = new Date();
		long duration = (end.getTime() - start.getTime());
		//Utils.Log("DEBUG: Query ("+query+") took "+duration+ " miliseconds");

	}
	/**
	 * This function executes the Update query from UpdateDatabase
	 * @param query
	 * @throws SQLException
	 */
	private void updateQuery(String query) throws SQLException
	{
		Date start = new Date();
		Statement statement = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
		statement.executeUpdate(query);
		Date end = new Date();
		long duration = (end.getTime() - start.getTime());
		System.out.println("DEBUG: Query took "+duration+ " miliseconds");		
	}
	/**
	 * Closes the current Database connection.
	 * @throws SQLException
	 */
	public void closeConnection() 
	{
		try {
			if(!conn.isClosed())
				conn.close();
			if(resultSet!=null)
				resultSet.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Used to query a database and access the resultant data in a ResultSet
	 * @param tableName The name of the database table to be accessed
	 * @param selectedFields A list of the values to be selected through the queries. <code>new ArrayList<String>();</code> 
	 * be passed to select all fields (*)
	 * @param queryParams A hashmap containing the selection filters in the format <code>[column_name]=[value]</code>
	 * @param customQuery Any additional parameters to be added after the Where clause. Could be a GROUP BY or ORDER BY
	 * @return The resulting data in  a ResultSet
	 * @throws SQLException
	 */
	public ResultSet getDatabase(String tableName,ArrayList<String> selectedFields, Map<String,Object> queryParams, String customQuery) throws SQLException
	{
		String queryBuilder = "";
		if(selectedFields.size()==0)
		{
			selectedFields.add("*");
		}

		queryBuilder += "SELECT "+Utils.Join(selectedFields,",")+" FROM "+tableName;

		if(queryParams != null && queryParams.size()>0)
		{
			queryBuilder += " WHERE ";
			for (Map.Entry<String,Object> entry : queryParams.entrySet()) {

				try{
					Float f = Float.parseFloat(entry.getValue().toString());
					queryBuilder += " "+entry.getKey() + " = "+entry.getValue()+" AND";
				}
				catch(Exception ex)
				{
					queryBuilder += " "+entry.getKey() + " = '"+entry.getValue()+"' AND";
				}

			}
			if(customQuery!= null && customQuery!="")
				queryBuilder += customQuery+" AND";
			queryBuilder = queryBuilder.substring(0,queryBuilder.length()-3);
		}
		//System.out.println(queryBuilder);
		return executeQuery(queryBuilder);
	}

	/**
	 * Used to update a single cell in a table in the database.
	 * @param tableName The name of the database table to be accessed
	 * @param selectedFields 
	 * @param queryParams A hashmap containing the selection filters in the format <code>[column_name]=[value]</code>
	 * @param selectionFieldValues A list of the values to be selected through the queries. <code>new ArrayList<String>();</code> 
	 * be passed to select all fields (*)
	 * @param newFieldsValues
	 * @throws SQLException
	 */
	public void updateDatabase(String tableName,Map<String,Object> selectionFieldValues, Map<String,Object> newFieldsValues) throws SQLException
	{
		String queryBuilder = "UPDATE "+tableName+" SET ";

		if(newFieldsValues.size()>0)
		{
			ArrayList<String> al = new ArrayList<String>();
			for (Map.Entry<String,Object> entry : newFieldsValues.entrySet()) {
				try{
					Float f = Float.parseFloat(entry.getValue().toString());
					al.add(entry.getKey()+" = "+entry.getValue());
				}
				catch(Exception ex)
				{
					al.add(entry.getKey()+" = '"+entry.getValue()+"'");
				}				
			}
			queryBuilder += Utils.Join(al, ",");
		}

		if(selectionFieldValues.size()>0)
		{
			queryBuilder += " WHERE ";
			for (Map.Entry<String,Object> entry : selectionFieldValues.entrySet()) {

				try{
					Float f = Float.parseFloat(entry.getValue().toString());
					queryBuilder += " "+entry.getKey() + " = "+entry.getValue()+" AND";
				}
				catch(Exception ex)
				{
					queryBuilder += " "+entry.getKey() + " = '"+entry.getValue()+"' AND";
				}

			}
			queryBuilder = queryBuilder.substring(0,queryBuilder.length()-3);
		}
		//Utils.Log(queryBuilder);
		updateQuery(queryBuilder);
	}
	/**
	 * Used to insert a new row in a table in the database.
	 * @param tableName The name of the table where the row has to be inserted
	 * @param fieldValues the hashmap of column names and values
	 * @throws SQLException
	 */
	public void insertDatabase(String tableName,Map<String,Object> fieldValues,Map<String,Object> nonStringfieldValues) throws SQLException
	{
		String queryBuilder = "INSERT INTO "+tableName;
		if(nonStringfieldValues==null) nonStringfieldValues = new HashMap<String, Object>();
		if(fieldValues.size()>0)
		{
			queryBuilder += " ("+Utils.Join(new ArrayList<String>(fieldValues.keySet()), ",");
			if(!nonStringfieldValues.isEmpty())
				queryBuilder +=","+Utils.Join(new ArrayList<String>(nonStringfieldValues.keySet()), ",");
			queryBuilder+= " ) VALUES ";
			ArrayList<String> ll = new ArrayList<String>();
			for (Iterator i = fieldValues.values().iterator(); i.hasNext();) {
				ll.add(i.next().toString());

			}
			ArrayList<String> ll2 = new ArrayList<String>();
			for (Iterator i = nonStringfieldValues.values().iterator(); i.hasNext();) {
				ll2.add(i.next().toString());

			}
			queryBuilder += " ('"+Utils.Join(ll, "','")+"'";
			if(!ll2.isEmpty())
				queryBuilder += ","+Utils.Join(ll2, ",");
			queryBuilder += ") ";
		}
		//Utils.Log(queryBuilder);

		executeNonQuery(queryBuilder);

	}
}

 
