/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.fourelementscapital.db.vo.PeerPackage;

/**
 * Abstract class implements methods related to R Function
 */
public abstract class RFunctionDB extends AbstractTeamOrgDB {

	public static final int FUNCTION_TYPE_NORMAL=0;
	public static final int FUNCTION_TYPE_CLASS=1;
	public static final int FUNCTION_TYPE_METHOD=2;
	
	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_R_FUNCTION);
	}		
	
	/**
	 * Get tag table 
	 * @return "tags "
	 */		
	public String getTagTable() { return "tags "; } 
	
	/**
	 * Get tag item table 
	 * @return "function_tags"
	 */		
	public String getTagItemTable() { return "function_tags"; }
	
	/**
	 * Get tag item id 
	 * @return "function_id"
	 */			
	public String getTagItemId() { return "function_id";}
	
	/**
	 * Get tag item follow table 
	 * @return "function_followtags"
	 */			
	public String getTagItemFollowTable() { return "function_followtags";}
	
	/**
	 * Get R function
	 * @param function_id function id
	 * @return r function
	 * @throws Exception
	 */		
	public abstract Map getRFunction(int function_id) throws Exception;
	
	/**
	 * Get R functions by names
	 * @param functionnames function names
	 * @return r functions
	 * @throws Exception
	 */		
	public abstract Vector getRFunctions(Vector functionnames) throws Exception; 
	
	/**
	 * Get R functions by name
	 * @param function_name function name
	 * @return r function
	 * @throws Exception
	 */		
	public abstract Map getRFunction(String function_name) throws Exception; 
	
	/**
	 * Get folder name
	 * @param folder_id folder id
	 * @return folder name
	 * @throws Exception
	 */		
	public abstract String getFolderName(int folder_id) throws Exception;
	
	/**
	 * List R functions
	 * @return R functions
	 * @throws Exception
	 */		
	public abstract List listRFunctions() throws Exception;
	
	/**
	 * Update function order
	 * @param function_ids function ids
	 * @throws Exception
	 */		
	public abstract void updateFunctionOrder(int[] function_ids ) throws Exception;
	
	/**
	 * Update folder order
	 * @param folder_ids folder ids
	 * @throws Exception
	 */			
	public abstract void updateFolderOrder(int[] folder_ids ) throws Exception;
	
	/**
	 * List all R functions names
	 * @return R functions names
	 * @throws Exception
	 */		
	public abstract List listAllRFunctionNames() throws Exception;
	
	/**
	 * List all R script names
	 * @param folder_id folder id
	 * @return R script names
	 * @throws Exception
	 */		
	public abstract List listAllRScriptNames(int folder_id) throws Exception;
	
	/**
	 * List of functions by folder id
	 * @param folder_id folder id
	 * @return r functions
	 * @throws Exception
	 */			
	public abstract List<Map> listOfFunctions(int folder_id) throws Exception;
	
	/**
	 * List function groups
	 * @return function groups
	 * @throws Exception
	 */		
	public abstract List listFunctionGroups() throws Exception;
	
	/**
	 * List of folders
	 * @return folders
	 * @throws Exception
	 */			
	public abstract List listOfFolders( ) throws Exception;
	
	/**
	 * List of folders by group id
	 * @param group_id group id
	 * @return folders
	 * @throws Exception
	 */	
	public abstract List listOfFolders(String group_id) throws Exception;
	
	/**
	 * Get folder id by name
	 * @param folder_name folder name
	 * @return folder id
	 * @throws Exception
	 */			
	public abstract int getFolderID(String folder_name) throws Exception;
	
	/**
	 * Create R function
	 * @param folder_id folder id
	 * @param functionname function name
	 * @param path script file path
	 * @param func_type function type
	 * @return r function id
	 * @throws Exception
	 */		
	public abstract int createFunction(int folder_id,String functionname, String path,int func_type) throws Exception;
	
	/**
	 * Move folder
	 * @param folder_id folder id
	 * @param new_group_id new group id
	 * @throws Exception
	 */		
	public abstract void moveFolder(int folder_id, String new_group_id) throws Exception;
	
	/**
	 * Update R function folder
	 * @param function_id function id
	 * @param new_folder_id new folder id
	 * @throws Exception
	 */		
	public abstract void updateFunctionFolder(int function_id,int new_folder_id) throws Exception;
	
	/**
	 * Rename folder
	 * @param folder_id folder id
	 * @param foldername folder name
	 * @throws Exception
	 */			
	public abstract void renameFolder(int folder_id,String foldername) throws Exception;
	
	/**
	 * Update last 2 users tag
	 * @param function_id function id
	 * @param user_tagid user tag id
	 * @throws Exception
	 */	
	public abstract void updateLast2UsersTag(int function_id, int user_tagid) throws Exception;
	
	/**
	 * Get all tags
	 * @return tags
	 * @throws Exception
	 */		
	public abstract Vector getTags() throws Exception;
	 
	/**
	 * Get tag id by function id
	 * @param function_id function id
	 * @return tag ids
	 * @throws Exception
	 */	
	public abstract Vector getTagIds4Function(int function_id) throws Exception;
	
	/**
	 * Add if tag not exist
	 * @param tagname tag name
	 * @return tag id
	 * @throws Exception
	 */		
	 public abstract int addIfTagNotExist(String tagname) throws Exception;

	/**
	 * Add tag ids for function
	 * @param function_id function id
	 * @param tag_id tag id
	 * @throws Exception
	 */		 
	 public abstract void addTagIds4Function(int function_id, int tag_id) throws Exception;
	 
	/**
	 * Remove tag ids for function
	 * @param function_id function id
	 * @param tag_id tag id
	 * @throws Exception
	 */		 
	 public abstract void removeTagIds4Function(int function_id, int tag_id) throws Exception;
	
	/**
	 * Update lock
	 * @param function_id function id
	 * @param lockedby locked by
	 * @throws Exception
	 */		 
	 public abstract void updateLock(int function_id,String lockedby) throws Exception;
	 
	/**
	 * Set group order
	 * @param groupids group ids
	 * @throws Exception
	 */		 
	 public abstract void setGroupOrder( Vector groupids) throws Exception ;
	 
	/**
	 * Create folder
	 * @param folder folder name
	 * @param new_group_id new group id
	 * @return folder id
	 * @throws Exception
	 */		 
	 public abstract int createFolder(String folder, String new_group_id) throws Exception;
	 
	/**
	 * Remove folder
	 * @param folder_id folder id
	 * @throws Exception
	 */			 
	 public abstract void removeFolder(int folder_id) throws Exception;
	 
	/**
	 * Update function deleted
	 * @param function_id function id
	 * @throws Exception
	 */		 
	 public abstract void updateFunctionDeleted(int function_id) throws Exception;
	 
	/**
	 * Rename R function
	 * @param function_id function id
	 * @param functionname function name
	 * @param script_file script file path
	 * @throws Exception
	 */	 
	 public abstract void renameFunction(int function_id, String functionname, String script_file) throws Exception;
	 
	/**
	 * Delete R function
	 * @param function_id function id
	 * @throws Exception
	 */		 
	 public abstract void deleteFunction(int function_id) throws Exception ;
	 
	/**
	 * Update is wiki done
	 * @param function_id function id
	 * @param done is done
	 * @throws Exception
	 */			 
	 public abstract void updateWikiDone(int function_id, int done) throws Exception ;

	/**
	 * Get all function names ID
	 * @param wherecond where condition
	 * @return function names & ids
	 * @throws Exception
	 */			 
	 public abstract Map getAllFunctionNamesID(String wherecond) throws Exception;
	 
	/**
	 * Auto complete functions
	 * @param func_keyword function keyword
	 * @return r function
	 * @throws Exception
	 */	 
	 public abstract List autoCompleteFunctions(String func_keyword) throws Exception ;
	 
	/**
	 * Updated owner ID
	 * @param owner_id owner id
	 * @param function_id function id
	 * @throws Exception
	 */		 
	 public abstract void updatedOwnerIDNow(int owner_id,int function_id) throws Exception;
	 
	/**
	 * Get package info by package name
	 * @param pname package name
	 * @return package info
	 * @throws Exception
	 */			 
	 public abstract Map getPackageInfo(String pname) throws Exception;
	 
	/**
	 * Get default hierarchy depends ids by package name
	 * @param pname package name
	 * @return id
	 * @throws Exception
	 */		 
	 public abstract List getDefaultHierarchyDependsIds(String pname) throws Exception;
	 
	/**
	 * List related folder ids
	 * @param parent_folder parent folder id
	 * @return related folder ids
	 * @throws Exception
	 */		 
	 public abstract List listRelatedFolderIds(int parent_folder) throws Exception;
	 
	/**
	 * Update related folder ids
	 * @param folder_id folder id
	 * @param ids related folder ids
	 * @throws Exception
	 */			 
	 public abstract void updateRelatedFolderIds(int folder_id, List<Integer> ids) throws Exception;
	 
	/**
	 * Add tag for package
	 * @param tag_id tag id
	 * @param function_ids function ids
	 * @param make_owner_also is make owner also
	 * @throws Exception
	 */		 
	 public abstract void addTag4Package(int tag_id,int function_ids[],boolean make_owner_also) throws Exception;
	 
	/**
	 * Add notification tag for package
	 * @param tag_id tag id
	 * @param function_ids function ids
	 * @param add_also is add a new record also
	 * @throws Exception
	 */		 
	 public abstract void addNotificationTag4Package(int tag_id,int function_ids[],boolean add_also) throws Exception;
	 
	/**
	 * Remove tag for package
	 * @param tag_id tag id
	 * @param function_ids function ids
	 * @throws Exception
	 */		 
	 public abstract void removeTag4Package(int tag_id,int function_ids[]) throws Exception ;
	 
	/**
	 * Update peer package
	 * @param peername peer name
	 * @param plist peer package list
	 * @throws Exception
	 */		 
	 public abstract boolean updatePeerPackage(String peername, ArrayList<PeerPackage>plist) throws Exception;
	 
	/**
	 * Get R function for script file
	 * @param scriptfile script file
	 * @return r functions
	 * @throws Exception
	 */		 
	 public abstract Map getRFunctionForScriptFile(String scriptfile) throws Exception;
	 
	/**
	 * Update tag id by folder id
	 * @param folder_id folder id
	 * @param tag_ids tag ids
	 * @param removeExisting is remove existing
	 * @param tablename table name
	 * @throws Exception
	 */			 
	 public abstract void updateTagIds4Folder(int folder_id, List tag_ids,int removeExisting, String tablename) throws Exception ;
	 
	/**
	 * Get tag id by folder id
	 * @param folder_id folder id
	 * @param tablename table name
	 * @return tag ids
	 * @throws Exception
	 */		 
	 public abstract ArrayList getTagIds4Folder(int folder_id, String tablename)	throws Exception ;
	 
	/**
	 * Get theme tags by folder id
	 * @param folder_id folder id
	 * @return theme tags
	 * @throws Exception
	 */	 
	 public abstract List getThemeTags4Folder(int folder_id) throws Exception  ;
		
	/**
	 * Get folder theme by R function id
	 * @param r_function_id r function id
	 * @return theme
	 * @throws Exception
	 */	 
	 public abstract String getFolderThemeByRFunctionId(int r_function_id) throws Exception  ;

	 public abstract void syncLib(int counter, int theme, int wikiDone, String rScript) throws Exception  ;

	 public abstract void truncateFunctions() throws Exception ;
}


 