/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.fourelementscapital.db.vo.SchedulerTrigger;

/**
 * Abstract class implements methods related to scheduler
 */
public abstract class SchedulerDB extends AbstractTeamOrgDB {

	public static int REMOVE_BEFORE_UPDATE=1;
	public static int DONT_REMOVE_BEFORE_UPDATE=2;
	
	public static final int PEER_ASSOCIATION_HISTORY_ADDED=1;
	public static final int PEER_ASSOCIATION_HISTORY_REMOVED=0;
	public static final int PEER_ASSOCIATION_HISTORY_PEER_ACTIVE=1;
	public static final int PEER_ASSOCIATION_HISTORY_PEER_NOACTIVE=0;	
	
	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_BBSYNC);
	}
	
	/**
	 * Get tag table 
	 * @return "tags "
	 */		
	public String getTagTable() { return "tags "; } 
	
	/**
	 * Get tag item table 
	 * @return "scheduler_tags"
	 */		
	public String getTagItemTable() { return "scheduler_tags"; }
	
	/**
	 * Get tag item id 
	 * @return "scheduler_id"
	 */		
	public String getTagItemId() { return "scheduler_id";}
	
	/**
	 * Get tag item follow table 
	 * @return "scheduler_followtags"
	 */		
	public String getTagItemFollowTable() { return "scheduler_followtags";}
	
	/**
	 * List scheduler
	 * @return schedulers
	 * @throws Exception
	 */		
	public abstract List listScheduler() throws Exception;
	
	/**
	 * List scheduler join folder by task UID 
	 * @return scheduler taskUID join
	 * @throws Exception
	 */		
	public abstract List listSchedulerTaskUIDJoin() throws Exception;
	
	/**
	 * List scheduler by theme tags
	 * @param themetags theme tags
	 * @return scheduler
	 * @throws Exception
	 */		
	public abstract List listScheduler(List<String> themetags) throws Exception;
	
	/**
	 * List trashed scheduler
	 * @return scheduler
	 * @throws Exception
	 */		
	public abstract List listTrashedScheduler() throws Exception;
	
	/**
	 * Search scheduler by keyword
	 * @param ids ids
	 * @return scheduler
	 * @throws Exception
	 */		
	public abstract List searchScheduler(String ids) throws Exception;
	
	/**
	 * Check a scheduler whether it's deleted
	 * @param id id
	 * @return true if it's deleted
	 * @throws Exception
	 */		
	public abstract boolean isDeleted(int id) throws Exception;
	
	/**
	 * List all tasks by UID
	 * @param taskuid taskuid
	 * @return scheduler
	 * @throws Exception
	 */		
	public abstract List listAllTasksByUID(String taskuid) throws Exception;
	
	/**
	 * List scheduler item by id
	 * @param scheduler_id scheduler id
	 * @return scheduler
	 * @throws Exception
	 */		
	public abstract Map listSchedulerItem(long scheduler_id) throws Exception ;
	
	/**
	 * Search scheduler by keyword & tag id
	 * @param keyword keyword
	 * @param tagid tag id
	 * @return schedulers
	 * @throws Exception
	 */		
	public abstract List searchScheduler(String keyword, int tagid) throws Exception ;
	
	/**
	 * List of folders by taskuid
	 * @param taskuid taskuid
	 * @return folders
	 * @throws Exception
	 */	
	public abstract List listOfFolders(String taskuid) throws Exception;
	
	/**
	 * List of folders by theme tags
	 * @param themetags theme tags
	 * @return folders
	 * @throws Exception
	 */		
	public abstract List listofMyFolders(List<String> themetags) throws Exception ;
	
	/**
	 * Get taskuid from folder by folder id
	 * @param folder_id folder id
	 * @return taskuid
	 * @throws Exception
	 */		
	public abstract String getFolderTaskUID(int folder_id) throws Exception ;
	 
	/**
	 * Update task folder
	 * @param scheduler_id scheduler id
	 * @param folder_id folder id
	 * @throws Exception
	 */		
	public abstract void updateTaskFolder(int scheduler_id,int folder_id) throws Exception;
	
	/**
	 * Fix broken folders
	 * @throws Exception
	 */		
	public abstract void fixSchBrokenFolders() throws Exception;
	
	/**
	 * Move item to root or group
	 * @param scheduler_id scheduler id
	 * @param taskuid taskuid
	 * @param folder_id folder id
	 * @throws Exception
	 */		
	public abstract void moveItemToRootOrGroup(int scheduler_id, String taskuid, int folder_id) throws Exception;
	
	/**
	 * Rename folder
	 * @param taskuid task uid
	 * @param oldname old name
	 * @param newname new name
	 * @throws Exception
	 */		
	public abstract void renameFolder(String taskuid, String oldname, String newname) throws Exception;
	
	/**
	 * Delete folder
	 * @param folderid folder id
	 * @throws Exception
	 */		
	public abstract void deleteFolder(int folderid) throws Exception;
	
	/**
	 * Add folder
	 * @param taskuid taskuid
	 * @param foldername folder name
	 * @return generated folder id
	 * @throws Exception
	 */		
	public abstract long addFolder(String taskuid,String foldername) throws Exception;
	
	/**
	 * Delete scheduler (set deleted flag)
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */		
	public abstract void deleteScheduler(int scheduler_id) throws Exception;
	
	/**
	 * Put back scheduler (set deleted flag to NULL)
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */		
	public abstract void putBackScheduler(int scheduler_id) throws Exception ;
	
	/**
	 * Get scheduler by id
	 * @param scheduler_id scheduler id
	 * @return schedulers
	 * @throws Exception
	 */		
	public abstract Map getScheduler(int scheduler_id) throws Exception;
	
	/**
	 * Add scheduler log
	 * @param scheduler_id scheduler id
	 * @param start start date
	 * @param end end date
	 * @param timezone timezone
	 * @param status status
	 * @param messages messages
	 * @return generated scheduler log id
	 * @throws Exception
	 */		
	public abstract int addSchedulerLog(int scheduler_id, Date start, Date end,String timezone, String status,String messages) throws Exception;
	
	/**
	 * Update scheduler log message
	 * @param log_id log id
	 * @param messages messages
	 * @throws Exception
	 */		
	public abstract void updateSchedulerLogMsg(int log_id,String messages) throws Exception;
	
	/**
	 * List of queue logs by time
	 * @param sids_trids scheduler id & trigger time
	 * @param now now time
	 * @return scheduler queue logs
	 * @throws Exception
	 */		
	public abstract  List listOfQueueLogs(String sids_trids, long now) throws Exception;
	
	/**
	 * List of history queue logs by time & date query
	 * @param now now time
	 * @param datequery date query
	 * @return queue logs
	 * @throws Exception
	 */			
	public abstract  List listOfHistoryQueueLogs(long now, String datequery) throws Exception;
	
	/**
	 * List of last 15 queue logs by time & scheduler id
	 * @param now now time
	 * @param scheduler_id scheduler id
	 * @return scheduler last 15 queue logs
	 * @throws Exception
	 */		
	public abstract  List listOfLast15Logs(long now, int scheduler_id) throws Exception;
	
	/**
	 * Remove queue logs by time & scheduler id
	 * @param currentTime current time
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */		
	public abstract void removeQueueLog(long currentTime, int scheduler_id) throws Exception;
	
	/**
	 * Update queue log status by id
	 * @param id queue logs id
	 * @param status status
	 * @throws Exception
	 */		
	public abstract void updateQueueLogStatus(int id,String status, String computerName) throws Exception;
	
	/**
	 * Get host name
	 * @return host name
	 * @throws Exception
	 */		
	public abstract String getHostName() throws Exception;	
	
	/**
	 * Update queue log data
	 * @param data data
	 * @param connection_ids connection ids
	 * @throws Exception
	 */		
	public abstract void updateQueueLog(Collection data, Vector connection_ids, String computerName) throws Exception;
	
	/**
	 * Update queue log executed code
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param code code
	 * @throws Exception
	 */		
	public abstract void updateExecutedCodeQLog(int scheduler_id, long trigger_time, String code ) throws Exception;
	
	/**
	 * Get log messages
	 * @param log_id log id
	 * @return log messages
	 * @throws Exception
	 */		
	public abstract String getLogMessages(int log_id) throws Exception;
	
	/**
	 * Toggle active
	 * @param scheduler_id scheduler id
	 * @return is active
	 * @throws Exception
	 */		
	public abstract int toggleActive(int scheduler_id) throws Exception;
	
	/**
	 * Get id of add or update scheduler
	 * @param scheduler_id scheduler id
	 * @param data data
	 * @param taskUID taskUID 
	 * @throws Exception
	 */		
	public abstract Integer addOrUpdateSchedulerGetId(int scheduler_id,Map<String,String> data, String taskUID) throws Exception;
	
	/**
	 * Get R script of add or update scheduler
	 * @param thisid scheduler id
	 * @param data data
	 * @param shortname short name 
	 * @throws Exception
	 */		
	public abstract void addOrUpdateSchedulerInsertTaskdata(Integer thisid,Map<String,String> data, String shortname) throws Exception;
	
	/**
	 * Update queue log server start & end 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param start date start
	 * @param end date end
	 * @throws Exception
	 */		
	public abstract void updateServerStartEnd(int scheduler_id,long  trigger_time,Date start, Date end) throws Exception;
	
	/**
	 * List dependency
	 * @param ids scheduler ids
	 * @param d date
	 * @return queue logs
	 * @throws Exception
	 */		
	public abstract List listDependencyList(String ids, Date d) throws Exception;
	
	/**
	 * Get peer list
	 * @return peername
	 * @throws Exception
	 */			
	public abstract Vector getPeersList() throws Exception ;
	
	/**
	 * Update peers notes
	 * @param peername peer name
	 * @param notes notes
	 * @param friendlyname friendly name
	 * @param cmail contact mail
	 * @throws Exception
	 */		
	public abstract void updatePeersNotes(String peername,String notes,String friendlyname, String cmail) throws Exception;
	
	/**
	 * Update peers last online
	 * @param peername peer name
	 * @param time time
	 * @throws Exception
	 */	
	public abstract void updatePeersLastOnline(String peername,long time) throws Exception;
	
	/**
	 * Update peers active by peername
	 * @param peername peer name
	 * @param active is active
	 * @param user user
	 * @throws Exception
	 */		
	public abstract void updatePeersToggle(String peername,int active, String user) throws Exception;
	
	/**
	 * Get peers data
	 * @return peers data
	 * @throws Exception
	 */	
	public abstract Vector getPeersData() throws Exception ;
	
	/**
	 * Update peers list peername
	 * @param peername peer name
	 * @throws Exception
	 */		
	public abstract void updatePeersList(String peername) throws Exception;
	
	/**
	 * Remove queue logs by time & task types
	 * @param currentTime current time
	 * @param tasktypes task types
	 * @throws Exception
	 */		
	public abstract void removeQueueLogs(long currentTime, String tasktypes) throws Exception;
	
	/**
	 * Get queue log 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return scheduler queue logs
	 * @throws Exception
	 */	
	public abstract Map getQueueLog(int scheduler_id, long trigger_time) throws Exception;
	
	/**
	 * Get associated peers
	 * @param taskuid taskuid
	 * @return peer name
	 * @throws Exception
	 */		
	public abstract Vector getAssociatedPeers(String taskuid) throws Exception;
	
	/**
	 * Get associated available peers
	 * @param taskuid taskuid
	 * @return peer name
	 * @throws Exception
	 */		
	public abstract Vector getAssoAvailablePeers(String taskuid) throws Exception;
	
	/**
	 * Get peer friendly name
	 * @param peername peer name
	 * @return friendly name
	 * @throws Exception
	 */		
	public abstract String getPeerFriendlyName(String peername) throws Exception ;
	
	/**
	 * Get all peer association
	 * @return task peers
	 * @throws Exception
	 */		
	public abstract Vector getAllPeerAssociation() throws Exception ;

	/**
	 * Get last successful queue log by multi scheduler ids
	 * @param scheduler_ids scheduler ids
	 * @return queue log
	 * @throws Exception
	 */		
	public abstract Vector getLastSuccessfulQLogs(Vector scheduler_ids) throws Exception;
	
	/**
	 * Get last successful queue log
	 * @param scheduler_id scheduler id
	 * @return queue log
	 * @throws Exception
	 */		
	public abstract Map getLastSuccessfulQLog(int scheduler_id) throws Exception;
	
	/**
	 * List all tasks by folder id
	 * @param folder_id folder id
	 * @return schedulers
	 * @throws Exception
	 */		
	public abstract List folderTasks(int folder_id) throws Exception;
	
	/**
	 * Move tasks
	 * @param d_tuid new task uid
	 * @param d_fid new folder id
	 * @param s_tuid original task uid
	 * @param s_fid original folder id
	 * @throws Exception
	 */		
	public abstract void moveTasks(String d_tuid, int d_fid, String s_tuid, int s_fid) throws Exception;
	
	/**
	 * Add R script
	 * @param name name
	 * @param script script
	 * @param restart restart
	 * @return generated id
	 * @throws Exception
	 */		
	public abstract int addRScript(String name,String script,int restart) throws Exception;
	
	/**
	 * Get R script
	 * @param script_id script id
	 * @return R script
	 * @throws Exception
	 */	
	public abstract Map getRScript(int script_id) throws Exception;	
	
	/**
	 * Add R script log
	 * @param script_id script id
	 * @param peer peer
	 * @param status status
	 * @param start_time start time
	 * @param end_time end time
	 * @param message message
	 * @throws Exception
	 */		
	public abstract void addRScriptLog(int script_id,String peer, String status, Date start_time, Date end_time, String message) throws Exception;
	
	/**
	 * Get last 5 R script logs
	 * @return r script logs
	 * @throws Exception
	 */		
	public abstract Vector rScriptLast5Logs() throws Exception;
	
	/**
	 * List of queue logs by scheduler id & trigger time
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return scheduler queue logs
	 * @throws Exception
	 */		
	public abstract Map listOfQueueLog(int scheduler_id, long trigger_time ) throws Exception; 
	
	/**
	 * Update queue log host & start 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param start date start
	 * @param host host
	 * @throws Exception
	 */		
	public abstract void updateHostAndStarted(int scheduler_id,long  trigger_time,Date start, String host) throws Exception;

	/**
	 * Move items
	 * @param foldername folder name
	 * @param taskuid taskuid
	 * @param parentfolder parent folder
	 * @param targetuid target uid
	 * @param targetpath target path
	 * @return scheduler id
	 * @throws Exception
	 */		
	public abstract ArrayList<Integer> moveItems(String foldername, String taskuid, String parentfolder, String targetuid, String targetpath) throws Exception;
	
	/**
	 * Update calling another script field
	 * @param scheduler_id scheduler id
	 * @param calling_another_script is calling another script
	 * @throws Exception
	 */		
	public abstract void updateCallingAnotherScript(int scheduler_id, int calling_another_script /* either 1 or 0 */) throws Exception;  
	
	/**
	 * Add peer taskuid
	 * @param taskuid task uid
	 * @param peer peer
	 * @param user user
	 * @throws Exception
	 */		
	public abstract void addPeerTaskuid(String taskuid, String peer, String user) throws Exception;
	
	/**
	 * Delete peer taskuid
	 * @param taskuid task uid
	 * @param peer peer
	 * @param user user
	 * @throws Exception
	 */	
	public abstract void deletePeerTaskuid(String taskuid, String peer, String user) throws Exception;
	
	/**
	 * Set group order
	 * @param taskuids task uids
	 * @throws Exception
	 */		
	public abstract void setGroupOrder( Vector taskuids) throws Exception;
	
	/**
	 * Get group order
	 * @return task uid
	 * @throws Exception
	 */		
	public abstract Vector getGroupOrder( ) throws Exception;
	
	/**
	 * Get all groups
	 * @return groups
	 * @throws Exception
	 */		
	public abstract Vector getAllGroups() throws Exception ;
	
	/**
	 * Get active groups
	 * @return groups
	 * @throws Exception
	 */		
	public abstract Vector getActiveGroups() throws Exception ;
	
	/**
	 * Get groups by engine type
	 * @param enginetype engine type
	 * @return groups
	 * @throws Exception
	 */		
	public abstract Vector getGroups(String enginetype) throws Exception ;
	
	/**
	 * Add scheduler execution logs
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param date date
	 * @param messages messages
	 * @param repCode rep code
	 * @throws Exception
	 */		
	public abstract void addSchedulerExeLogs(int scheduler_id, long trigger_time, Date date,String messages,int repCode, String computerName) throws Exception;

	/**
	 * Get execution logs
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return execution logs
	 * @throws Exception
	 */		
	public abstract Vector getSchedulerExeLogs(int scheduler_id, long trigger_time ) throws Exception ;
	
	/**
	 * Update queue log status where status is null or '' 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param status status
	 * @throws Exception
	 */	
	public abstract void updateQueueNullStatus(int scheduler_id, long trigger_time,String status) throws Exception;
	
	/**
	 * Get trigger data by scheduler id
	 * @param scheduler_id scheduler id
	 * @return triggers
	 * @throws Exception
	 */		
	public abstract Vector getTriggerData(int scheduler_id) throws Exception;
	
	/**
	 * Set trigger data
	 * @param scheduler_id scheduler id
	 * @param arry triggers
	 * @throws Exception
	 */		
	public abstract void setTriggerData(int scheduler_id, SchedulerTrigger arry[]) throws Exception;
	
	/**
	 * Get trigger data by id
	 * @param row_id id
	 * @return triggers
	 * @throws Exception
	 */		
	public abstract Map getOneRowTriggerData(long row_id) throws Exception;
	
	/**
	 * Add peer thread status
	 * @param peername peer name
	 * @param queuename queuename
	 * @param taskuid taskuid
	 * @param allowedthread allowed thread
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */		
	public abstract void addPeerThreadStatus(String peername,String queuename, String taskuid, int allowedthread, int scheduler_id) throws Exception ;
	
	/**
	 * Remove peer thread status
	 * @param peername peer name
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */	
	public abstract void removePeerThreadStatus(String peername, int scheduler_id) throws Exception ;
	
	/**
	 * Remove all peer thread status
	 * @param peername peer name
	 * @throws Exception
	 */		
	public abstract void removeAllPeerThreadStatus(String peername) throws Exception ;
	
	/**
	 * Get available peers
	 * @param peers peers
	 * @param taskuid taskuid
	 * @return peer names
	 * @throws Exception
	 */			
	public abstract Vector getAvailablePeers(String peers /* "'rhino'" */, String taskuid /* "'peer1','peer2'" */) throws Exception;
	
	/**
	 * Get max duration in last 50 execution
	 * @param scheduler_id scheduler id
	 * @return max duration
	 * @throws Exception
	 */		
	public abstract long getMaxDurationInLast50Exec(int scheduler_id) throws Exception;
	
	
	//public abstract int get20RecentMaxDuration(int scheduler_id, long trigger_time) throws Exception;
	
	/**
	 * Get timeout criteria in ms
	 * @param scheduler_id scheduler id
	 * @return timeout criteria in ms
	 * @throws Exception
	 */		
	public abstract long getTimeoutCriteriaInMs(int scheduler_id) throws Exception;
	
	/**
	 * Check whether name is exist
	 * @param name name
	 * @param scheduler_id scheduler id
	 * @return true if name is exist
	 * @throws Exception
	 */	
	public abstract boolean isNameExisting(String name,int scheduler_id) throws Exception;
	
	/**
	 * Get DB log summary
	 * @param con_ids connection ids
	 * @return DB log summary
	 * @throws Exception
	 */	
	public abstract Map getDBLogSummary(String con_ids) throws Exception ;
	
	//public abstract Map getDBLogSummary(int scheduler_id,long trigger_time) throws Exception ;
	
	/**
	 * Get data log history
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return data log history
	 * @throws Exception
	 */		
	public abstract Vector getDataLogHistory(int scheduler_id, long trigger_time) throws Exception;
	
	/**
	 * Update alert type
	 * @param alert_type alert type
	 * @param scheduler_id scheduler id
	 * @return true
	 * @throws Exception
	 */		
	public abstract boolean updateAlertType(String alert_type,int scheduler_id) throws Exception ;
	
	//public abstract Vector getThemeTags(int scheduler_id) throws Exception;
	
	/**
	 * Get error message even null
	 * @param scheduler_id scheduler id
	 * @param trig_time trigger time
	 * @return error message
	 * @throws Exception
	 */		
	public abstract String getErrorMessageEvenNull(int scheduler_id,long trig_time) throws Exception;
	
	/**
	 * Update last 2 users tag
	 * @param scheduler_id scheduler id
	 * @param user_tagid user tag id
	 * @throws Exception
	 */	
	 public abstract void updateLast2UsersTag(int scheduler_id, int user_tagid) throws Exception;
	 
	/**
	 * Add if tag not exist
	 * @param tagname tag name
	 * @return generated tag id
	 * @throws Exception
	 */	 
	 public abstract int addIfTagNotExist(String tagname) throws Exception;
	 
	/**
	 * Get task event actions
	 * @param scheduler_id scheduler id
	 * @param trig_time trigger time
	 * @return task event actions
	 * @throws Exception
	 */		
	public abstract Map getTaskEventActions(int scheduler_id,long trig_time) throws Exception;
	
	//public abstract void pushAlarmMessage(String message) throws Exception;	
	
	/**
	 * Push alarm message
	 * @param db_name_trading_ref database name trading ref
	 * @param message message
	 * @throws Exception
	 */		
	public abstract void pushAlarmMessage(String db_name_trading_ref, String message) throws Exception;		
	
	/**
	 * Get scheduler dependency
	 * @param scheduler_id scheduler id
	 * @return scheduler ids
	 * @throws Exception
	 */		
	public abstract List<Integer> getDependsThis(int scheduler_id) throws Exception;
	
	/**
	 * Get scheduler depend to
	 * @param scheduler_id scheduler id
	 * @return scheduler ids
	 * @throws Exception
	 */		
	public abstract List<Integer> getDependsTo(int scheduler_id) throws Exception ;	

	/**
	 * Purge scheduler
	 * @param scheduler_id scheduler id
	 * @throws Exception
	 */		
	public abstract void purgeScheduler(int scheduler_id) throws Exception;
	
	/**
	 * Add edit logs
	 * @param scheduler_id scheduler id
	 * @param user user
	 * @param message message
	 * @throws Exception
	 */	
	public abstract void addEditLogs(int scheduler_id, String user,String message) throws Exception;
	
	/**
	 * Get edit logs
	 * @param scheduler_id scheduler id
	 * @return edit logs
	 * @throws Exception
	 */		
	public abstract List getEditLogs(int scheduler_id) throws Exception;
	
	/**
	 * Get inject code from queue log
	 * @param sid_triggertime scheduler id & trigger time
	 * @return inject code
	 * @throws Exception
	 */		
	public abstract String getInjectCode4QLog(String sid_triggertime) throws Exception;

	/**
	 * Update timeout settings
	 * @param data timeout setting data
	 * @throws Exception
	 */		
	public abstract void updateTimeoutSettings(Map data) throws Exception;
	
	/**
	 * Get timeout settings
	 * @return timeout settings
	 * @throws Exception
	 */		
	public abstract Map getTimeoutSettings() throws Exception ;
	
	/**
	 * Get last 10 minutes suspected failure
	 * @return suspected failure
	 * @throws Exception
	 */		
	public abstract ArrayList getLast10minuteSuspectedFailure() throws Exception; 
	
	/**
	 * Update queue log response code 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param code response code
	 * @throws Exception
	 */		
	public abstract void updateResponseCode(int scheduler_id, long trigger_time,int code) throws Exception;	
	
	/**
	 * Get peers list by engine type
	 * @param enginetypes engine types
	 * @return peers list
	 * @throws Exception
	 */		
	public abstract ArrayList getPeersList4Engine(List<String> enginetypes) throws Exception;
	
	/**
	 * Get console message 
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return console message
	 * @throws Exception
	 */	
	public abstract String getConsoleMsg(int scheduler_id, long trigger_time) throws Exception;
	
	/*
	 *Get console message from 2nd Scheduler 
	 * @param scheduler_id scheduler id
	 * @return console message
	 * @throws Exception 
	 */
	public abstract String getConsoleMsgFrom2ndScheduler(int scheduler_id) throws Exception;
	
	/**
	 * Get single col data
	 * @param query
	 * @return data
	 * @throws Exception
	 */	
	public abstract List getSingleColData(String query) throws Exception;
	
	/**
	 * User pref save or update
	 * @param login login
	 * @param ky key
	 * @param val value
	 * @throws Exception
	 */	
	public abstract void userPrefSaveOrUpdate(String login, String ky, String val) throws Exception ;
	
	/**
	 * Get user preference by login
	 * @param login login
	 * @return user preferences
	 * @throws Exception
	 */		
	public abstract Map userPrefGetAll(String login) throws Exception; 
	
	/**
	 * Get auto complete XHR
	 * @param keyword keyword
	 * @return auto complete XHR
	 * @throws Exception
	 */		
	public abstract List getAutoCompleteXHR(String keyword) throws Exception;
	
	/**
	 * Get owner theme
	 * @param scheduler_id scheduler id
	 * @return tag name
	 * @throws Exception
	 */		
	public abstract String getOwnerTheme(int scheduler_id) throws Exception;
	
	/**
	 * Update is wiki done
	 * @param scheduler_id scheduler id
	 * @param done is wiki done
	 * @throws Exception
	 */		
	public abstract void updateWikiDone(int scheduler_id, int done) throws Exception;
	
	/**
	 * Get last 2 days failed by user's themes
	 * @param themes user's themes
	 * @return queue logs
	 * @throws Exception
	 */		
	public abstract List getFailedLast2Days(List<String> themes) throws Exception  ;
	
	/**
	 * Get last 50 peer action history
	 * @param peername peer name
	 * @return action history
	 * @throws Exception
	 */		
	public abstract List getLast50PeerActHistory(String peername) throws Exception ;
	
	/**
	 * Get last 50 task action history
	 * @param taskuid task uid
	 * @return action history
	 * @throws Exception
	 */		
	public abstract List getLast50TaskActHistory(String taskuid) throws Exception ;
	
	/**
	 * Update queue log status by scheduler id & trigger time
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param status status
	 * @throws Exception
	 */		
	public abstract void updateQueueLogStatus(int scheduler_id, long trigger_time,String status, String computerName) throws Exception ;	
	
	/**
	 * Check whether execution log is exist by repcode
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @param repcode rep code
	 * @return true if exist
	 * @throws Exception
	 */			
	public abstract boolean execLogsRepcodeExist(int scheduler_id, long trigger_time, int repcode) throws Exception ;
	
	/**
	 * Is there any execution logs in last 3 minutes
	 * @param scheduler_id scheduler id
	 * @param trigger_time trigger time
	 * @return true if execution logs found
	 * @throws Exception
	 */		
	public abstract boolean isAnyExecLogsInLast3Mins(int scheduler_id, long trigger_time) throws Exception;

	/**
	 * Get query of top 20 id & name by name
	 * @param keyword keyword
	 * @return query
	 * @throws Exception
	 */		
	public abstract String getTop20IdNameByNameQuery(String keyword) throws Exception;
	
	/**
	 * Get query of top 20 id & name by id
	 * @param keyword keyword
	 * @return query
	 * @throws Exception
	 */		
	public abstract String getTop20IdNameByIdQuery(String keyword) throws Exception;
	
	/**
	 * Get query of history queue logs date
	 * @param scheduelr_id scheduler id
	 * @param stat status
	 * @return query
	 */	
	public abstract String getHistoryQueueLogsDateQuery(long scheduelr_id, String stat) throws Exception;
	
	/**
	 * Get completed signal from scheduler_exeplanlogs table
	 * @param scheduler_id scheduler id
	 * @return query
	 * @throws Exception
	 */	
	public abstract String checkCompletedSignalFromPeer(int scheduler_id) throws Exception;

}

 
