/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import com.fourelementscapital.db.vo.ValueObject;

/**
 * Abstract class implements methods related to utility
 */
public abstract class UtilDB extends SuperDB {

	/**
	 * List commodities
	 * @param con connection
	 * @return commodities
	 * @throws Exception
	 */		
	public abstract List listCommodities(Connection con) throws Exception ;
	
	/**
	 * List commodities with filter
	 * @param con connection
	 * @param filter filter
	 * @return commodities
	 * @throws Exception
	 */			
	public abstract List listCommoditiesWithFilter(Connection con,String filter) throws Exception;
	
	/**
	 * List first letters of commodities
	 * @param con connection
	 * @return commodities
	 * @throws Exception
	 */		
	public abstract List listCommoditiesFirstLetters(Connection con) throws Exception ;
	
	/**
	 * Get unique fields of commodity field original
	 * @param con connection
	 * @return commodity field original
	 * @throws Exception
	 */	
	public abstract List uniqueFields(Connection con) throws Exception ;
	
	/**
	 * List all tables by commodity field original & commodity
	 * @param con connection
	 * @param field_arr field arr
	 * @param commodity_arr commodity arr
	 * @return tables
	 * @throws Exception
	 */		
	public abstract List<Map> listAllTables4Fields(Connection con, String field_arr, String commodity_arr ) throws Exception;
	
	/**
	 * List all orphaned assets
	 * @param con connection
	 * @return tables
	 * @throws Exception
	 */		
	public abstract List<Map> listAllOrphanedAssets(Connection con ) throws Exception;
	
	/**
	 * Remove all orphaned assets
	 * @param con connection
	 * @throws Exception
	 */		
	public abstract void removeAllOrphanedAssets(Connection con) throws Exception;
	
	/**
	 * List all table strings by commodities
	 * @param con connection
	 * @param fieldtype field type
	 * @param commodities commodities
	 * @throws Exception
	 */		
	public abstract List<String> listAllTables4Commodities(Connection con, String fieldtype,Vector<String> commodities) throws Exception;
	
	/**
	 * Get field original
	 * @param con connection
	 * @param tablename table name
	 * @return commodity field original
	 * @throws Exception
	 */	
	public abstract String getFieldOriginal(Connection con,String tablename) throws Exception;
	
	/**
	 * List all table value objects by commodities
	 * @param con connection
	 * @param fieldtype field type
	 * @param commodities commodities
	 * @throws Exception
	 */		
	public abstract Vector<ValueObject> listAllTables4Commodities2(Connection con, String fieldtype,Vector<String> commodities) throws Exception;
	
	/**
	 * List all contract titles
	 * @param con connection
	 * @param jointables join tables
	 * @return contract titles
	 * @throws Exception
	 */
	public abstract TreeMap<String,Vector> listAllContractTitles(Connection con, String jointables) throws Exception;
 
	/**
	 * List all contract titles
	 * @param con connection
	 * @param jointables join tables
	 * @return contract titles
	 * @throws Exception
	 */	 
	public abstract TreeMap<String,Vector> listAllContractTitles2(Connection con, String jointables) throws Exception;
	
	/**
	 * List all contract titles in lower case
	 * @param con connection
	 * @param jointables join tables
	 * @return contract titles
	 * @throws Exception
	 */	
	public abstract List<String> listAllContractTitles2LCase(Connection con, String jointables) throws Exception;
	
	/**
	 * Remove commodity and fields
	 * @param con connection
	 * @param commodity commodity
	 * @return true
	 * @throws Exception
	 */		
	public abstract boolean removeCommodityAndFields(Connection con,String commodity) throws Exception;
	
	/**
	 * Get raw data by table name, contract & date query
	 * @param datequery date query
	 * @param con connection
	 * @param tablename table name
	 * @param commodity commodity
	 * @return raw data
	 * @throws Exception
	 */
	public abstract Vector getRawData(String datequery,Connection con,String tablename, String commodity) throws Exception;
	
	/**
	 * Get record count by contract & commodity
	 * @param con connection
	 * @param commodity commodity
	 * @param contract contract
	 * @return record count
	 * @throws Exception
	 */	
	public abstract List getRecordCount4Contracts(Connection con, String commodity, String contract ) throws Exception;
	
	/**
	 * Get raw data
	 * @param datequery date query
	 * @param con connection
	 * @param tablename table name
	 * @param commodity commodity
	 * @return raw data
	 * @throws Exception
	 */	
	public abstract Map getRawData3(String datequery,Connection con,String tablename, String commodity) throws Exception;
	
	/**
	 * Get raw data by contract 
	 * @param datequery date query
	 * @param con connection
	 * @param fieldname field name
	 * @param contrat contrat
	 * @param commodity commodity
	 * @return raw data
	 * @throws Exception
	 */			
	public abstract Map getRawData2Contract(String datequery,Connection con,String fieldname, String contrat,String commodity) throws Exception;
	
	/**
	 * Delete raw data
	 * @param con connection
	 * @param tablename table name
	 * @param data data
	 * @throws Exception
	 */		
	public abstract void deleteRawData(Connection con,String tablename,Map<String,String> data) throws Exception;
	
	/**
	 * Update raw data
	 * @param con connection
	 * @param tablename table name
	 * @param olddata old data
	 * @param newdata new data
	 * @throws Exception
	 */		
	public abstract void updateRawData(Connection con,String tablename,Map<String,String> olddata,Map<String,String> newdata) throws Exception;
	
	/**
	 * Remove field table
	 * @param con connection
	 * @param ftable field table
	 * @return true
	 * @throws Exception
	 */		
	public abstract boolean removeFieldTable(Connection con,String ftable) throws Exception;
	
	/**
	 * Rename field table
	 * @param con connection
	 * @param ftable field table
	 * @param fieldoriginal field original
	 * @return true if success
	 * @throws Exception
	 */		
	public abstract boolean renameFieldTable(Connection con,String ftable, String fieldoriginal) throws Exception;
	
	/**
	 * List field tables for admin
	 * @param con connection
	 * @param mtable commodity
	 * @return field tables
	 * @throws Exception
	 */		
	public abstract List listFieldTables4Admin(Connection con,String mtable) throws Exception;
	
	/**
	 * List field tables
	 * @param con connection
	 * @param mtable commodity
	 * @return field tables
	 * @throws Exception
	 */		
	public abstract List listFieldTables(Connection con,String mtable) throws Exception;
	
	/**
	 * List all unique fields
	 * @param con connection
	 * @return unique fields
	 * @throws Exception
	 */	
	public abstract List  listAllUniqueFields(Connection con) throws Exception;
	
	/**
	 * List all unique fields by commodity
	 * @param con connection
	 * @param commodity commodity
	 * @return all unique fields
	 * @throws Exception
	 */	
	public abstract List  listAllUniqueFields(Connection con, String commodity) throws Exception;
	
	/**
	 * Delete contracts
	 * @param con connection
	 * @param field_tablename field table name
	 * @param contract contract
	 * @throws Exception
	 */		
	public abstract void deleteContracts(Connection con, String field_tablename, String contract) throws Exception;
	
	/**
	 * Get unique contracts
	 * @param con connection
	 * @param field_tablename field table name
	 * @return contracts
	 * @throws Exception
	 */		
	public abstract Vector getUniqueContracts(Connection con, String field_tablename) throws Exception;
	
	/**
	 * List all commodities and fields
	 * @param con connection
	 * @return commodities and fields
	 * @throws Exception
	 */		
	public abstract Map listAllCommoditiesAndFields(Connection con) throws Exception;
	
	/**
	 * Last sync update
	 * @param con connection
	 * @throws Exception
	 */	
	public abstract void LastSyncUpdate(Connection con) throws Exception;
	
	/**
	 * Last sync days
	 * @param con connection
	 * @param today date
	 * @return sync date
	 * @throws Exception
	 */		
	public abstract int lastSyncDays(Connection con,Date today) throws Exception;
	
	/**
	 * Last sync date
	 * @param con connection
	 * @return sync date
	 * @throws Exception
	 */	
	public abstract  Date lastSyncDate(Connection con) throws Exception;

	/**
	 * Get daily strategy data
	 * @param con connection
	 * @param date date
	 * @param contractname contract name
	 * @param fields fields
	 * @return strategy data
	 * @throws Exception
	 */		
	 public abstract TreeMap dailyStrategyData(Connection con,String date, String contractname, String fields) throws Exception;
	  
	/**
	 * List saved chart
	 * @param con connection
	 * @return saved charts
	 * @throws Exception
	 */		  
	 public abstract TreeMap listSavedChart(Connection con) throws Exception;

	/**
	 * Delete saved chart
	 * @param con connection
	 * @param id id
	 * @throws Exception
	 */		  
	 public abstract void deleteSavedChart(Connection con, int id) throws Exception;
	 
	/**
	 * Get saved chart item
	 * @param con connection
	 * @param id id
	 * @return saved chart
	 * @throws Exception
	 */		  
	 public abstract Map getSavedChartItem(Connection con, int id) throws Exception;
	  
	/**
	 * Add saved chart
	 * @param con connection
	 * @param data data
	 * @throws Exception
	 */		  
	 public abstract void savedChart(Connection con,Map data) throws Exception;

	/**
	 * Show old connections
	 * @param minutesbefore minutes before
	 * @param computername computer name
	 * @return SPID
	 * @throws Exception
	 */
	 public abstract ArrayList showOldConnections(int minutesbefore, String computername) throws Exception;
	 
	/**
	 * Kill connections
	 * @param conids connection ids
	 * @throws Exception
	 */		 
	 public abstract void killConnections(List<Integer> conids) throws Exception ;
	 
 	 
}

 