/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.fileutils;

import java.util.Map;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class provide function to replace 'place holder' in template with values.
 * Example : 'Name : [[name]], Address : [[address]].' will become 'Name : Jon, Address : Bali.'
 */
public class StringPlaceHolder {
	
	public static final String PATTERN = "\\[\\[(.*?)\\]\\]"; // example : [[name]]
	public static final String PATTERN_ATTR = "\\\"\\[\\[(.*?)\\]\\]\\\""; // example : \"[[name]]\"

	/**
	 * Private constructor
	 */	
	private StringPlaceHolder() {}

	
	/**
	 * Get elements (strings) in the template that will be replaced by another string. Use custom pattern.
	 * @param template Template string
	 * @param patternAttrStr pattern to find attribute
	 * @return attributes
	 */		
	public static Vector<String> getAttributePH(final String template, final String patternAttrStr){
		Vector<String> v=new Vector<String>();
	    final StringBuffer sb = new StringBuffer();
	    final Pattern pattern =
	        Pattern.compile(patternAttrStr, Pattern.DOTALL);
	    final Matcher matcher = pattern.matcher(template);
	    while(matcher.find()){
	        final String key = matcher.group(1);
	        if(!v.contains(key)){
	        	v.add(key);
	        }
	    }
	    return v;
	}
	
	
	/**
	 * Overloading method of getAttributePH() with default pattern attribute values.
	 * @param template Template string
	 * @return attributes
	 */		
	public static Vector<String> getAttributePH(final String template) {
    	return getAttributePH(template, StringPlaceHolder.PATTERN_ATTR);
    }
	

	/**
	 * Get elements (strings) in the template that will be replaced by another string. Use custom pattern.
	 * The elements in the result found by getAttributePH() will be removed.
	 * @param template Template string
	 * @param patternAttrStr pattern to find attribute
	 * @return elements
	 */	
	public static Vector<String> getElementPH(final String template, final String patternStr){
			Vector<String> v=new Vector<String>();
		    final StringBuffer sb = new StringBuffer();
		    final Pattern pattern =
		        Pattern.compile(patternStr, Pattern.DOTALL);
		    final Matcher matcher = pattern.matcher(template);
		    while(matcher.find()){
		        final String key = matcher.group(1);
		        if(!v.contains(key)){
		        	v.add(key);
		        }
		    }
		    v.removeAll(getAttributePH(template));
		    return v;
	}
	
	
	/**
	 * Overloading method of getElementPH() with default pattern values.
	 * @param template Template string
	 * @return elements
	 */		
	public static Vector<String> getElementPH(final String template) {
		return getElementPH(template, StringPlaceHolder.PATTERN);
    }  
	
	
	/**
	 * Replace strings found by pattern in template with new input values. Use custom pattern.
	 * @param template Template string
	 * @param values Replacement values
	 * @param patternStr pattern to be replaced
	 * @return string from template with replaced strings
	 */		
	public static String parse(final String template, final Map values, final String patternStr){

	    final StringBuffer sb = new StringBuffer();
	    final Pattern pattern =
	        Pattern.compile(patternStr, Pattern.DOTALL);
	    final Matcher matcher = pattern.matcher(template);
	    while(matcher.find()){
	        final String key = matcher.group(1);
	        if( values.get(key)!=null){
		        final String replacement = values.get(key).toString();
		        if(replacement == null){
		        	matcher.appendReplacement(sb, "");
		        }else{
		        	matcher.appendReplacement(sb, replacement);
		        }
	        }else{
	        	matcher.appendReplacement(sb, "");
	        }
	    }
	    matcher.appendTail(sb);
	    return sb.toString();
	}
	
	/**
	 * Overloading method of parse() with default pattern values.
	 * @param template Template string
	 * @param values Replacement values
	 * @return string from template with replaced strings
	 */		
    public static String parse(final String template, final Map values){ 
    	return parse(template, values, StringPlaceHolder.PATTERN);
    }	

    
}

 
